# -*- encoding: utf-8 -*-
# Copyright (c) 2005 Canonical
# Copyright (c) 2004 Conectiva, Inc.
#
# Written by Gustavo Niemeyer <niemeyer@conectiva.com>
#
# This file is part of Smart Package Manager.
#
# Smart Package Manager is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# Smart Package Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Smart Package Manager; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from smart.backends.rpm.synthesis import URPMISynthesisLoader
from smart.backends.rpm.header import URPMILoader
from smart.util.filetools import getFileDigest, compareFiles
from smart.const import SUCCEEDED, FAILED, ALWAYS, NEVER
from smart.channel import PackageChannel
from smart import *
import posixpath
import re
import os
import string

class URPMIChannel(PackageChannel):

    def __init__(self, baseurl, hdlurl, infourl, *args):
        super(URPMIChannel, self).__init__(*args)
        self._baseurl = baseurl
        if hdlurl:
            if hdlurl[0] != "/" and ":/" not in hdlurl:
                self._hdlurl = posixpath.join(self._baseurl, hdlurl)
            else:
                self._hdlurl = hdlurl
        else:
            self._hdlurl = posixpath.join(self._baseurl, "hdlist.cz")
        if infourl:
            if infourl[0] != "/" and ":/" not in infourl:
                self._infourl = posixpath.join(self._baseurl, infourl)
            else:
                self._infourl = infourl
        else:
            self._infourl = posixpath.join(self._baseurl, "info.xml.lzma")

        self._compareurl = self._hdlurl

    def getCacheCompareURLs(self):
        return [self._compareurl]

    def getFetchSteps(self):
        return 3

    def fetch(self, fetcher, progress):

        fetcher.reset()

        if self.getAlias().startswith("urpmisync-") and sysconf.get("sync-urpmi-medialist"):
            urpmiDir = sysconf.get("sync-urpmi-mediadir")
            urpmiMediaDir = posixpath.join(urpmiDir, self.getName())

            # We'll force migration to new layout of urpmi media rather than
            # implementing support for old layout
            if not posixpath.exists(urpmiMediaDir):
                os.mkdir(urpmiMediaDir)
            # Here we'll also force moving of existing data to new layout and
            # also to get rid of any old leftovers..
            for filename in ["changelog.%s.xml.lzma", "files.%s.xml.lzma", \
                    "info.%s.xml.lzma", "MD5SUM.%s", "hdlist.%s.cz", \
                    "synthesis.hdlist.%s.cz"]:
                filename %= self.getName()
                oldfile = posixpath.join(urpmiDir, filename)
                if not os.path.exists(oldfile):
                    continue
                tmp = filename.split("."+self.getName())
                newfile = posixpath.join(urpmiMediaDir, string.join(tmp, ""))
                if compareFiles(oldfile, newfile) or \
                        os.path.getmtime(oldfile) < os.path.getmtime(oldfile) or\
                        os.path.getsize == 0:
                    os.unlink(oldfile)
                else:
                    os.rename(oldfile, newfile)

            fetcher.setLocalDir(urpmiMediaDir)
            fetcher.setLocalPathPrefix("")

        self._compareurl = self._hdlurl

        hdlbaseurl, basename = os.path.split(self._hdlurl)

        md5url = posixpath.join(hdlbaseurl, "MD5SUM")
        item = fetcher.enqueue(md5url)
        fetcher.run(progress=progress)

        hdlmd5 = None
        failed = item.getFailedReason()
        if not failed:
            self._compareurl = md5url
            digest = getFileDigest(item.getTargetPath())

            if digest == self._digest:
                progress.add(2)
                return True

            basename = posixpath.basename(self._hdlurl)
            for line in open(item.getTargetPath()):
                line = line.strip()
                if line:
                    md5, name = line.split()
                    if name == basename:
                        hdlmd5 = md5
                        break

        fetcher.reset()
        hdlitem = fetcher.enqueue(self._hdlurl, md5=hdlmd5, uncomp=False)

        fetcher.run(progress=progress)

        if hdlitem.getStatus() == FAILED:
            hdfailed = hdlitem.getFailedReason()
            if fetcher.getCaching() is NEVER:
                # Try reading reconfig.urpmi (should give new path)
                fetcher.reset()
                reconfigurl = posixpath.join(hdlbaseurl, "reconfig.urpmi")
                reconfigitem = fetcher.enqueue(reconfigurl)
                fetcher.run(progress=progress)
                if reconfigitem.getStatus() == FAILED:
                    refailed = reconfigitem.getFailedReason()
                    if fetcher.getCaching() is NEVER:
                        lines = [_("Failed acquiring information for '%s':") % self,
                            u"%s: %s" % (hdlitem.getURL(), hdfailed),
                            u"%s: %s" % (reconfigitem.getURL(), refailed)]
                        raise Error, "\n".join(lines)
                    return False
                else:
                    # Need to inject "/" at the end to avoid buggy urls
                    if not hdlbaseurl.endswith("/"): hdlbaseurl += "/"
                    for line in open(reconfigitem.getTargetPath()):
                        if line.startswith("#"): pass
                        elif line:
                            splitline = line.split()
                            arch = os.uname()[4]
                            if arch == "i686": arch = "i586"
                            reconfarch = re.sub("\$ARCH", arch, splitline[1])
                            reconfpath = re.sub(splitline[0] + "$", reconfarch, hdlbaseurl)
                            sysconf.set(("channels", self.getAlias(), \
                                        "baseurl"), reconfpath)
                            self._hdlurl = os.path.join(reconfpath, basename)
                    return self.fetch(fetcher, progress)
            return False
        else:
            localpath = hdlitem.getTargetPath()
            digestpath = None
            
            # Packdrakeng archive has a 64 byte trailer at end, if present
            # we're using hdlist.cz
            localhdl = open(localpath)
            localhdl.seek(0,2)
            if localhdl.tell() > 64:
                localhdl.seek(-64,2)
            trailer = localhdl.read()
            localhdl.close()

            if self._infourl and trailer[:3] + trailer[-3:] != "cz[]cz":
                infoitem = fetcher.enqueue(self._infourl, uncomp=False)
                fetcher.run(progress=progress)
            else:
                infoitem = None

            if infoitem and infoitem.getStatus() == SUCCEEDED:
                if self._compareurl == self._hdlurl:
                    self._compareurl = infourl
                    digestpath = localpath
                infopath = infoitem.getTargetPath()
            else:
                infopath = None
                if self._compareurl == self._hdlurl:
                    digestpath = localpath
            if digestpath:
                digest = getFileDigest(digestpath)
                if digest == self._digest:
                    return True
            self.removeLoaders()

            if len(trailer) >= 64 and trailer[:3] + trailer[-3:] == "cz[]cz":
                uncompressor = fetcher.getUncompressor()
                uncomphandler = uncompressor.getHandler(localpath)
                uncomphandler.uncompress(localpath)
                localpath = localpath[:-3]
                loader = URPMILoader(localpath, self._baseurl, None)
            else:
                loader = URPMISynthesisLoader(localpath, self._baseurl, infopath)
                                
            loader.setChannel(self)
            self._loaders.append(loader)

        self._digest = digest

        return True

def create(alias, data):
    return URPMIChannel(data["baseurl"],
                        data["hdlurl"],
                        data["infourl"],
                        data["type"],
                        alias,
                        data["name"],
                        data["manual"],
                        data["removable"],
                        data["priority"])

# vim:ts=4:sw=4:et
